@@ -43,13 +43,13 @@
 #include <fcntl.h>
 #include <limits.h>
 #include <poll.h>
-#include <sched.h>
 #include <signal.h>
 #include <stdarg.h>
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
 #include <sys/mman.h>
+#include <sys/mount.h>
 #include <sys/stat.h>
 #include <sys/types.h>
 #include <unistd.h>
@@ -95,13 +95,12 @@ strong_alias(env_unset_environment,	slurm_env_unset_environment);
 #define ENV_BUFSIZE (256 * 1024)
 #define MAX_ENV_STRLEN (32 * 4096)	/* Needed for CPU_BIND and MEM_BIND on
 					 * SGI systems with huge CPU counts */
-
-#define STACK_SIZE (1024 * 1024) /* For clone() syscall. */
-
 typedef struct {
 	char *cmdstr;
 	int *fildes;
-	int mode;
+	bool perform_mount;
+	int rlimit;
+	char **tmp_env;
 	const char *username;
 } child_args_t;
 
@@ -142,7 +141,7 @@ static char **
 _extend_env(char ***envp)
 {
 	char **ep;
-	size_t newcnt = (xsize (*envp) / sizeof (char *)) + 1;
+	size_t newcnt = PTR_ARRAY_SIZE(*envp) + 1;
 
 	*envp = xrealloc (*envp, newcnt * sizeof (char *));
 
@@ -401,6 +400,7 @@ int setup_env(env_t *env, bool preserve_env)
 	    (env->stepid != SLURM_INTERACTIVE_STEP)) {
 		char *str_verbose, *str_bind1 = NULL, *str_bind2 = NULL;
 		char *str_bind_list, *str_bind_type = NULL, *str_bind = NULL;
+		bool append_cpu_bind = false;
 
 		unsetenvp(env->env, "SLURM_CPU_BIND");
 		unsetenvp(env->env, "SLURM_CPU_BIND_LIST");
@@ -424,21 +424,23 @@ int setup_env(env_t *env, bool preserve_env)
 
 		if (env->cpu_bind_type & CPU_BIND_NONE) {
 			str_bind2 = "none";
-		} else if (env->cpu_bind_type & CPU_BIND_RANK) {
-			str_bind2 = "rank";
 		} else if (env->cpu_bind_type & CPU_BIND_MAP) {
 			str_bind2 = "map_cpu:";
+			append_cpu_bind = true;
 		} else if (env->cpu_bind_type & CPU_BIND_MASK) {
 			str_bind2 = "mask_cpu:";
+			append_cpu_bind = true;
 		} else if (env->cpu_bind_type & CPU_BIND_LDRANK) {
 			str_bind2 = "rank_ldom";
 		} else if (env->cpu_bind_type & CPU_BIND_LDMAP) {
 			str_bind2 = "map_ldom:";
+			append_cpu_bind = true;
 		} else if (env->cpu_bind_type & CPU_BIND_LDMASK) {
 			str_bind2 = "mask_ldom:";
+			append_cpu_bind = true;
 		}
 
-		if (env->cpu_bind)
+		if (env->cpu_bind && append_cpu_bind)
 			str_bind_list = env->cpu_bind;
 		else
 			str_bind_list = "";
@@ -459,12 +461,22 @@ int setup_env(env_t *env, bool preserve_env)
 		} else
 			str_bind_type = xstrdup("");
 
-		if (setenvf(&env->env, "SLURM_CPU_BIND", "%s", str_bind)) {
+		/*
+		 * Don't set SLURM_CPU_BIND or SLURM_CPU_BIND_LIST in the
+		 * environment if they are too long. These are informational
+		 * for the user and don't merit an error in the log if they
+		 * can't be set, so avoid calling setenvf().
+		 */
+		if (strlen(str_bind) >= MAX_ENV_STRLEN)
+			debug("Not setting SLURM_CPU_BIND: value too long");
+		else if (setenvf(&env->env, "SLURM_CPU_BIND", "%s", str_bind)) {
 			error("Unable to set SLURM_CPU_BIND");
 			rc = SLURM_ERROR;
 		}
-		if (setenvf(&env->env, "SLURM_CPU_BIND_LIST", "%s",
-			    str_bind_list)) {
+		if (strlen(str_bind_list) >= MAX_ENV_STRLEN)
+			debug("Not setting SLURM_CPU_BIND_LIST: value too long");
+		else if (setenvf(&env->env, "SLURM_CPU_BIND_LIST", "%s",
+				 str_bind_list)) {
 			error("Unable to set SLURM_CPU_BIND_LIST");
 			rc = SLURM_ERROR;
 		}
@@ -486,7 +498,6 @@ int setup_env(env_t *env, bool preserve_env)
 	if (env->mem_bind_type && (env->stepid != SLURM_INTERACTIVE_STEP)) {
 		char *str_verbose, *str_bind_type = NULL, *str_bind_list;
 		char *str_prefer = NULL, *str_bind = NULL;
-		char *str_bind_sort = NULL;
 
 		if (env->batch_flag) {
 			unsetenvp(env->env, "SBATCH_MEM_BIND");
@@ -498,7 +509,6 @@ int setup_env(env_t *env, bool preserve_env)
 			unsetenvp(env->env, "SLURM_MEM_BIND");
 			unsetenvp(env->env, "SLURM_MEM_BIND_LIST");
 			unsetenvp(env->env, "SLURM_MEM_BIND_PREFER");
-			unsetenvp(env->env, "SLURM_MEM_BIND_SORT");
 			unsetenvp(env->env, "SLURM_MEM_BIND_TYPE");
 			unsetenvp(env->env, "SLURM_MEM_BIND_VERBOSE");
 		}
@@ -521,9 +531,6 @@ int setup_env(env_t *env, bool preserve_env)
 			str_bind_type = "local";
 		}
 
-		if (env->mem_bind_type & MEM_BIND_SORT)
-			str_bind_sort = "sort";
-
 		if (env->mem_bind)
 			str_bind_list = env->mem_bind;
 		else
@@ -557,12 +564,6 @@ int setup_env(env_t *env, bool preserve_env)
 				error("Unable to set SBATCH_MEM_BIND_PREFER");
 				rc = SLURM_ERROR;
 			}
-			if (str_bind_sort &&
-			    setenvf(&env->env, "SBATCH_MEM_BIND_SORT", "%s",
-				    str_bind_sort)) {
-				error("Unable to set SBATCH_MEM_BIND_SORT");
-				rc = SLURM_ERROR;
-			}
 			if (setenvf(&env->env, "SBATCH_MEM_BIND_TYPE", "%s",
 				    str_bind_type)) {
 				error("Unable to set SBATCH_MEM_BIND_TYPE");
@@ -589,12 +590,6 @@ int setup_env(env_t *env, bool preserve_env)
 				error("Unable to set SLURM_MEM_BIND_PREFER");
 				rc = SLURM_ERROR;
 			}
-			if (str_bind_sort &&
-			    setenvf(&env->env, "SLURM_MEM_BIND_SORT", "%s",
-				    str_bind_sort)) {
-				error("Unable to set SLURM_MEM_BIND_SORT");
-				rc = SLURM_ERROR;
-			}
 			if (setenvf(&env->env, "SLURM_MEM_BIND_TYPE", "%s",
 				    str_bind_type)) {
 				error("Unable to set SLURM_MEM_BIND_TYPE");
@@ -620,6 +615,12 @@ int setup_env(env_t *env, bool preserve_env)
 		rc = SLURM_ERROR;
 	}
 
+	if (env->oom_kill_step != NO_VAL16 &&
+	    setenvf(&env->env, "SLURM_OOM_KILL_STEP", "%u", env->oom_kill_step)) {
+		error("Unable to set SLURM_OOM_KILL_STEP environment");
+		rc = SLURM_ERROR;
+	}
+
 	if (env->slurmd_debug
 	    && setenvf(&env->env, "SLURMD_DEBUG", "%d", env->slurmd_debug)) {
 		error("Can't set SLURMD_DEBUG environment variable");
@@ -856,6 +857,15 @@ int setup_env(env_t *env, bool preserve_env)
 		}
 	}
 
+	if (env->tls_cert) {
+		if (setenvf(&env->env, "SLURM_SRUN_TLS_CERT", "%s",
+			    env->tls_cert)) {
+			error("%s: can't set SLURM_SRUN_TLS_CERT env variable",
+			      __func__);
+			rc = SLURM_ERROR;
+		}
+	}
+
 	return rc;
 }
 
@@ -880,7 +890,7 @@ extern char *uint16_array_to_str(int array_len, const uint16_t *array)
 {
 	int i;
 	int previous = 0;
-	char *sep = ",";  /* seperator */
+	char *sep = ",";  /* separator */
 	char *str = xstrdup("");
 
 	if (array == NULL)
@@ -924,7 +934,7 @@ extern char *uint32_compressed_to_str(uint32_t array_len,
 				      const uint32_t *array_reps)
 {
 	int i;
-	char *sep = ","; /* seperator */
+	char *sep = ","; /* separator */
 	char *str = xstrdup("");
 
 	if (!array || !array_reps)
@@ -979,7 +989,7 @@ extern int env_array_for_job(char ***dest,
 	char *dist = NULL;
 	char *key, *value;
 	slurm_step_layout_t *step_layout = NULL;
-	int i, rc = SLURM_SUCCESS;
+	int i, new_cpt, rc = SLURM_SUCCESS;
 	slurm_step_layout_req_t step_layout_req;
 	uint16_t cpus_per_task_array[1];
 	uint32_t cpus_task_reps[1];
@@ -995,10 +1005,10 @@ extern int env_array_for_job(char ***dest,
 
 	if (het_job_offset < 1) {
 		env_array_overwrite_fmt(dest, "SLURM_JOB_ID", "%u",
-					alloc->job_id);
+					alloc->step_id.job_id);
 	}
-	env_array_overwrite_het_fmt(dest, "SLURM_JOB_ID", het_job_offset,
-				    "%u", alloc->job_id);
+	env_array_overwrite_het_fmt(dest, "SLURM_JOB_ID", het_job_offset, "%u",
+				    alloc->step_id.job_id);
 	env_array_overwrite_het_fmt(dest, "SLURM_JOB_NAME", het_job_offset,
 				    "%s", desc->name);
 	env_array_overwrite_het_fmt(dest, "SLURM_JOB_NUM_NODES", het_job_offset,
@@ -1045,7 +1055,7 @@ extern int env_array_for_job(char ***dest,
 
 	/* OBSOLETE, but needed by MPI, do not remove */
 	env_array_overwrite_het_fmt(dest, "SLURM_JOBID", het_job_offset, "%u",
-				    alloc->job_id);
+				    alloc->step_id.job_id);
 	env_array_overwrite_het_fmt(dest, "SLURM_NNODES", het_job_offset, "%u",
 				    step_layout_req.num_hosts);
 	env_array_overwrite_het_fmt(dest, "SLURM_NODELIST", het_job_offset, "%s",
@@ -1182,17 +1192,35 @@ extern int env_array_for_job(char ***dest,
 					    het_job_offset,
 					    "%d", desc->num_tasks);
 	}
-	if (desc->bitflags & JOB_CPUS_SET) {
+
+	new_cpt = slurm_opt_get_tres_per_task_cpu_cnt(alloc->tres_per_task);
+	if (new_cpt) {
+		env_array_overwrite_het_fmt(dest, "SLURM_CPUS_PER_TASK",
+					    het_job_offset, "%d", new_cpt);
+	} else if (desc->bitflags & JOB_CPUS_SET) {
 		env_array_overwrite_het_fmt(dest, "SLURM_CPUS_PER_TASK",
 					    het_job_offset, "%d",
 					     desc->cpus_per_task);
 	}
+
+	if (alloc->tres_per_task) {
+		env_array_overwrite_het_fmt(dest, "SLURM_TRES_PER_TASK",
+					    het_job_offset, "%s",
+					    alloc->tres_per_task);
+	}
+
 	if (desc->ntasks_per_node && (desc->ntasks_per_node != NO_VAL16)) {
 		env_array_overwrite_het_fmt(dest, "SLURM_NTASKS_PER_NODE",
 					    het_job_offset, "%d",
 					     desc->ntasks_per_node);
 	}
 
+	if (alloc->segment_size) {
+		env_array_overwrite_het_fmt(dest, "SLURM_JOB_SEGMENT_SIZE",
+					    het_job_offset, "%u",
+					    alloc->segment_size);
+	}
+
 	return rc;
 }
 
@@ -1271,7 +1299,8 @@ env_array_for_batch_job(char ***dest, const batch_job_launch_msg_t *batch,
 	env_array_overwrite_fmt(dest, "SLURM_CLUSTER_NAME", "%s",
 	                        slurm_conf.cluster_name);
 
-	env_array_overwrite_fmt(dest, "SLURM_JOB_ID", "%u", batch->job_id);
+	env_array_overwrite_fmt(dest, "SLURM_JOB_ID", "%u",
+				batch->step_id.job_id);
 	env_array_overwrite_fmt(dest, "SLURM_JOB_NUM_NODES", "%u",
 				step_layout_req.num_hosts);
 	if (batch->array_task_id != NO_VAL) {
@@ -1295,7 +1324,8 @@ env_array_for_batch_job(char ***dest, const batch_job_launch_msg_t *batch,
 		env_array_overwrite_fmt(dest, "HOSTNAME", "%s", node_name);
 
 	/* OBSOLETE, but needed by MPI, do not remove */
-	env_array_overwrite_fmt(dest, "SLURM_JOBID", "%u", batch->job_id);
+	env_array_overwrite_fmt(dest, "SLURM_JOBID", "%u",
+				batch->step_id.job_id);
 	env_array_overwrite_fmt(dest, "SLURM_NNODES", "%u",
 				step_layout_req.num_hosts);
 	env_array_overwrite_fmt(dest, "SLURM_NODELIST", "%s", batch->nodes);
@@ -1313,6 +1343,9 @@ env_array_for_batch_job(char ***dest, const batch_job_launch_msg_t *batch,
 	if (getenvp(*dest, "SLURM_CPUS_PER_TASK"))
 		env_array_overwrite_fmt(dest, "SLURM_CPUS_PER_TASK", "%u",
 					cpus_per_task);
+	if (batch->tres_per_task)
+		env_array_overwrite_fmt(dest, "SLURM_TRES_PER_TASK", "%s",
+					batch->tres_per_task);
 
 	if (step_layout_req.num_tasks) {
 		env_array_overwrite_fmt(dest, "SLURM_NTASKS", "%u",
@@ -1430,14 +1463,36 @@ env_array_for_step(char ***dest,
 		   uint16_t launcher_port,
 		   bool preserve_env)
 {
-	char *tmp, *tpn;
+	char *cluster_name, *tmp, *tpn;
 	uint32_t node_cnt, task_cnt;
 
 	if (!step || !launch)
 		return;
 
+	if (working_cluster_rec && working_cluster_rec->name)
+		cluster_name = working_cluster_rec->name;
+	else
+		cluster_name = slurm_conf.cluster_name;
+	env_array_overwrite_fmt(dest, "SLURM_CLUSTER_NAME", "%s", cluster_name);
+
 	node_cnt = step->step_layout->node_cnt;
-	env_array_overwrite_fmt(dest, "SLURM_STEP_ID", "%u", step->job_step_id);
+	env_array_overwrite_fmt(dest, "SLURM_STEP_ID", "%u",
+				step->step_id.step_id);
+
+	if (launch->cred && launch->cred->arg &&
+	    launch->cred->arg->job_account) {
+		env_array_overwrite_fmt(dest, "SLURM_JOB_ACCOUNT", "%s",
+					launch->cred->arg->job_account);
+	}
+
+	if (launch->cred && launch->cred->arg) {
+		tmp = gid_to_string_or_null(launch->cred->arg->gid);
+		if (tmp) {
+			env_array_overwrite_fmt(dest, "SLURM_JOB_GROUP", "%s",
+						tmp);
+			xfree(tmp);
+		}
+	}
 
 	if (launch->het_job_node_list) {
 		tmp = launch->het_job_node_list;
@@ -1484,7 +1539,8 @@ env_array_for_step(char ***dest,
 	}
 
 	/* OBSOLETE, but needed by some MPI implementations, do not remove */
-	env_array_overwrite_fmt(dest, "SLURM_STEPID", "%u", step->job_step_id);
+	env_array_overwrite_fmt(dest, "SLURM_STEPID", "%u",
+				step->step_id.step_id);
 	if (!preserve_env) {
 		env_array_overwrite_fmt(dest, "SLURM_NNODES", "%u", node_cnt);
 		env_array_overwrite_fmt(dest, "SLURM_NTASKS", "%u", task_cnt);
@@ -1499,7 +1555,7 @@ env_array_for_step(char ***dest,
 }
 
 /*
- * Enviroment variables set elsewhere
+ * Environment variables set elsewhere
  * ----------------------------------
  *
  * Set by slurmstepd:
@@ -1856,23 +1912,6 @@ void env_array_merge_slurm_spank(char ***dest_array, const char **src_array)
 	xfree(value);
 }
 
-/*
- * Strip out trailing carriage returns and newlines
- */
-static void _strip_cr_nl(char *line)
-{
-	int len = strlen(line);
-	char *ptr;
-
-	for (ptr = line+len-1; ptr >= line; ptr--) {
-		if (*ptr=='\r' || *ptr=='\n') {
-			*ptr = '\0';
-		} else {
-			return;
-		}
-	}
-}
-
 /* Return the net count of curly brackets in a string
  * '{' adds one and '}' subtracts one (zero means it is balanced).
  * Special case: return -1 if no open brackets are found */
@@ -1888,6 +1927,101 @@ static int _bracket_cnt(char *value)
 	return count;
 }
 
+/*
+ * Load user environment from a specified file or file descriptor.
+ *
+ * This will read in a user specified file or fd, that is invoked
+ * via the --export-file option in sbatch. The NAME=value entries must
+ * be NULL separated to support special characters in the environment
+ * definitions.
+ */
+char **env_array_from_file(const char *fname)
+{
+	char *buf = NULL, *ptr = NULL, *eptr = NULL;
+	char *value, *p;
+	char **env = NULL;
+	char name[256];
+	int buf_size = BUFSIZ, buf_left;
+	int file_size = 0, tmp_size;
+	int separator = '\0';
+	int fd;
+
+	if (!fname)
+		return NULL;
+
+	/*
+	 * If file name is a numeric value, then it is assumed to be a
+	 * file descriptor.
+	 */
+	fd = (int)strtol(fname, &p, 10);
+	if ((*p != '\0') || (fd < 3) || (fd > sysconf(_SC_OPEN_MAX)) ||
+	    (fcntl(fd, F_GETFL) < 0)) {
+		fd = open(fname, O_RDONLY);
+		if (fd == -1) {
+			error("Could not open user environment file %s", fname);
+			return NULL;
+		}
+		verbose("Getting environment variables from %s", fname);
+	} else
+		verbose("Getting environment variables from fd %d", fd);
+
+	/*
+	 * Read in the user's environment data.
+	 */
+	buf = xmalloc(buf_size);
+	buf_left = buf_size;
+	while ((tmp_size = read(fd, &buf[file_size], buf_left))) {
+		if (tmp_size < 0) {
+			if (errno == EINTR)
+				continue;
+			error("read(environment_file): %m");
+			break;
+		}
+
+		if (buf_left <= tmp_size) {
+			buf_size += BUFSIZ;
+			xrealloc(buf, buf_size);
+		}
+
+		file_size += tmp_size;
+		buf_left = buf_size - file_size;
+		if (buf_left < 0) {
+			error("%s: We don't have a large enough buffer.",
+			      __func__);
+			break;
+		}
+	}
+	close(fd);
+
+	/*
+	 * Parse the buffer into individual environment variable names
+	 * and build the environment.
+	 */
+	env   = env_array_create();
+	value = xmalloc(ENV_BUFSIZE);
+	for (ptr = buf; ; ptr = eptr+1) {
+		eptr = strchr(ptr, separator);
+		if ((ptr == eptr) || (eptr == NULL))
+			break;
+		if (_env_array_entry_splitter(ptr, name, sizeof(name),
+					      value, ENV_BUFSIZE) &&
+		    (!_discard_env(name, value))) {
+			/*
+			 * Unset the SLURM_SUBMIT_DIR if it is defined so
+			 * that this new value does not get overwritten
+			 * in the subsequent call to env_array_merge().
+			 */
+			if (xstrcmp(name, "SLURM_SUBMIT_DIR") == 0)
+				unsetenv(name);
+			env_array_overwrite(&env, name, value);
+		}
+	}
+	xfree(buf);
+	xfree(value);
+
+	return env;
+}
+
 int env_array_to_file(const char *filename, const char **env_array,
 		      bool newline)
 {
@@ -1928,97 +2062,49 @@ rwfail:
 	return rc;
 }
 
-/*
- * Load user environment from a cache file located in
- * <state_save_location>/env_username
- */
-static char **_load_env_cache(const char *username)
-{
-	char fname[PATH_MAX];
-	char *line, name[256], *value;
-	char **env = NULL;
-	FILE *fp;
-	int i;
-
-	i = snprintf(fname, sizeof(fname), "%s/env_cache/%s",
-		     slurm_conf.state_save_location, username);
-	if (i < 0) {
-		error("Environment cache filename overflow");
-		return NULL;
-	}
-	if (!(fp = fopen(fname, "r"))) {
-		error("Could not open user environment cache at %s: %m",
-			fname);
-		return NULL;
-	}
-
-	verbose("Getting cached environment variables at %s", fname);
-	env = env_array_create();
-	line  = xmalloc(ENV_BUFSIZE);
-	value = xmalloc(ENV_BUFSIZE);
-	while (1) {
-		if (!fgets(line, ENV_BUFSIZE, fp))
-			break;
-		_strip_cr_nl(line);
-		if (_env_array_entry_splitter(line, name, sizeof(name),
-					      value, ENV_BUFSIZE) &&
-		    (!_discard_env(name, value))) {
-			if (value[0] == '(') {
-				/* This is a bash function.
-				 * It may span multiple lines */
-				while (_bracket_cnt(value) > 0) {
-					if (!fgets(line, ENV_BUFSIZE, fp))
-						break;
-					_strip_cr_nl(line);
-					if ((strlen(value) + strlen(line)) >
-					    (ENV_BUFSIZE - 2))
-						break;
-					strcat(value, "\n");
-					strcat(value, line);
-				}
-			}
-			env_array_overwrite(&env, name, value);
-		}
-	}
-	xfree(line);
-	xfree(value);
-
-	fclose(fp);
-	return env;
-}
-
 static int _child_fn(void *arg)
 {
 	char **tmp_env = NULL;
-	int devnull;
+	int devnull, fd = 3;
 	child_args_t *child_args = arg;
 	char *cmdstr;
 	const char *username;
 
 	username = child_args->username;
 	cmdstr = child_args->cmdstr;
-	tmp_env = env_array_create();
-	env_array_overwrite(&tmp_env, "ENVIRONMENT", "BATCH");
+	tmp_env = child_args->tmp_env;
 
-	if ((devnull = open("/dev/null", O_RDONLY)) == -1)
-		error("%s: open(/dev/null): %m", __func__);
-
-	dup2(devnull, STDIN_FILENO);
-	dup2(child_args->fildes[1], STDOUT_FILENO);
-	dup2(devnull, STDERR_FILENO);
-	closeall(3);
-
-	if (child_args->mode == 1)
-		execle(SUCMD, "su", username, "-c", cmdstr, NULL, tmp_env);
-	else if (child_args->mode == 2)
-		execle(SUCMD, "su", "-", username, "-c", cmdstr, NULL, tmp_env);
-	else {	/* Default system configuration */
-#ifdef LOAD_ENV_NO_LOGIN
-		execle(SUCMD, "su", username, "-c", cmdstr, NULL, tmp_env);
-#else
-		execle(SUCMD, "su", "-", username, "-c", cmdstr, NULL, tmp_env);
-#endif
+#if !defined(__APPLE__) && !defined(__FreeBSD__) && !defined(__NetBSD__)
+	/*
+	 * Setting propagation and mounting our own /proc for this namespace.
+	 * This is done to ensure that this cloned process and its children
+	 * have coherent /proc contents with their virtual PIDs.
+	 * Check _clone_env_child to see namespace flags used in clone.
+	 */
+	if (child_args->perform_mount) {
+		if (mount("none", "/proc", NULL, MS_PRIVATE|MS_REC, NULL))
+			_exit(1);
+		if (mount("proc", "/proc", "proc",
+			  MS_NOSUID|MS_NOEXEC|MS_NODEV, NULL))
+			_exit(1);
 	}
+#endif
+
+	if ((devnull = open("/dev/null", O_RDWR)) != -1) {
+		dup2(devnull, STDIN_FILENO);
+		dup2(devnull, STDERR_FILENO);
+	}
+	dup2(child_args->fildes[1], STDOUT_FILENO);
+
+	/* slow close all fds */
+	while (fd < child_args->rlimit)
+		close(fd++);
+
+#ifdef LOAD_ENV_NO_LOGIN
+	execle(SUCMD, "su", username, "-c", cmdstr, NULL, tmp_env);
+#else
+	execle(SUCMD, "su", "-", username, "-c", cmdstr, NULL, tmp_env);
+#endif
 	if (devnull >= 0)	/* Avoid Coverity resource leak notification */
 		(void) close(devnull);
 
@@ -2029,6 +2115,7 @@ static int _child_fn(void *arg)
 static int _clone_env_child(child_args_t *child_args)
 {
 	char *child_stack;
+	int rc = 0;
 	child_stack = mmap(NULL, STACK_SIZE, PROT_READ | PROT_WRITE,
 			   MAP_PRIVATE | MAP_ANONYMOUS | MAP_STACK, -1, 0);
 	if (child_stack == MAP_FAILED) {
@@ -2044,53 +2131,109 @@ static int _clone_env_child(child_args_t *child_args)
 	 * Killing the 'child' pid will kill all the namespace, since in the
 	 * namespace, this 'child' is pid 1.
 	 */
-	return clone(_child_fn, child_stack + STACK_SIZE,
-		     (SIGCHLD|CLONE_NEWPID), child_args);
+	rc = clone(_child_fn, child_stack + STACK_SIZE,
+		   (SIGCHLD|CLONE_NEWPID|CLONE_NEWNS), child_args);
+	/* Memory deallocated only in parent address space, child unaffected */
+	if (munmap(child_stack, STACK_SIZE))
+		error("%s: failed to munmap child stack: %m", __func__);
+	return rc;
+}
+
+static bool _ns_path_disabled(const char *ns_path)
+{
+	FILE *fp = NULL;
+	size_t line_sz = 0;
+	ssize_t nbytes = 0;
+	int ns_value;
+	char *line = NULL;
+	bool ns_disabled = false;
+
+	/* We will assume not having these files as having no limits. */
+	fp = fopen(ns_path, "r");
+	if (!fp) {
+		debug2("%s: could not open %s, assuming no pid namespace limits. Reason: %m",
+		       __func__, ns_path);
+	} else {
+		nbytes = getline(&line, &line_sz, fp);
+		if (nbytes < 0) {
+			debug2("%s: could not read contents of %s. Assuming no namespace limits. Reason: %m",
+			       __func__, ns_path);
+		} else if (nbytes == 0) {
+			debug2("%s: read 0 bytes from %s. Assuming no namespace limits",
+			       __func__, ns_path);
+		} else {
+			ns_value = xstrntol(line, NULL, nbytes, 10);
+			if (ns_value == 0)
+				ns_disabled = true;
+		}
+		fclose(fp);
+		free(line);
+		line = NULL;
+	}
+
+	return ns_disabled;
+}
+
+/*
+ * Returns a boolean indicating if the required namespaces for the clone
+ * calls are disabled. This is performed by checking the contents of
+ * "/proc/sys/max_[mnt|pid]_namespaces" and ensuring they are not 0.
+ */
+static bool _ns_disabled()
+{
+	static int disabled = -1;
+	char *pid_ns_path = "/proc/sys/user/max_pid_namespaces";
+	char *mnt_ns_path = "/proc/sys/user/max_mnt_namespaces";
+
+	if (disabled != -1)
+		return disabled;
+
+	disabled = false;
+
+	if (_ns_path_disabled(pid_ns_path) ||
+	    _ns_path_disabled(mnt_ns_path))
+		disabled = true;
+
+	return disabled;
 }
 #endif
 
 /*
  * Return an array of strings representing the specified user's default
- * environment variables following a two-prongged approach.
- * 1. Execute (more or less): "/bin/su - <username> -c /usr/bin/env"
+ * environment variables:
+ *    Execute (more or less): "/bin/su - <username> -c /usr/bin/env"
  *    Depending upon the user's login scripts, this may take a very
  *    long time to complete or possibly never return
- * 2. Load the user environment from a cache file. This is used
- *    in the event that option 1 times out.  This only happens if no_cache isn't
- *    set.  If it is set then NULL will be returned if the normal load fails.
  *
- * timeout value is in seconds or zero for default (2 secs)
- * mode is 1 for short ("su <user>"), 2 for long ("su - <user>")
+ * timeout value is in seconds or zero for default (120 secs)
  * On error, returns NULL.
  *
  * NOTE: The calling process must have an effective uid of root for
  * this function to succeed.
  */
-char **env_array_user_default(const char *username, int timeout, int mode,
-			      bool no_cache)
+char **env_array_user_default(const char *username)
 {
 	char *line = NULL, *last = NULL, name[PATH_MAX], *value, *buffer;
 	char **env = NULL;
 	char *starttoken = "XXXXSLURMSTARTPARSINGHEREXXXX";
 	char *stoptoken  = "XXXXSLURMSTOPPARSINGHEREXXXXX";
-	char cmdstr[256], *env_loc = NULL;
+	char *cmdstr = NULL, *env_loc = NULL;
 	char *stepd_path = NULL;
 	int fildes[2], found, fval, len, rc, timeleft;
 	int buf_read, buf_rem, config_timeout;
+	int timeout = DEFAULT_GET_ENV_TIMEOUT;
 	pid_t child;
 	child_args_t child_args = {0};
 	struct timeval begin, now;
 	struct pollfd ufds;
 	struct stat buf;
+	struct rlimit rlim;
 
 	if (geteuid() != (uid_t)0) {
 		error("SlurmdUser must be root to use --get-user-env");
 		return NULL;
 	}
 
-	if (!slurm_conf.get_env_timeout)	/* just read directly from cache */
-		return _load_env_cache(username);
-
 	if (stat(SUCMD, &buf))
 		fatal("Could not locate command: "SUCMD);
 	if (stat("/bin/echo", &buf))
@@ -2105,10 +2248,11 @@ char **env_array_user_default(const char *username, int timeout, int mode,
 		env_loc = "/usr/bin/env";
 	else
 		fatal("Could not locate command: env");
-	snprintf(cmdstr, sizeof(cmdstr),
-		 "/bin/echo; /bin/echo; /bin/echo; "
-		 "/bin/echo %s; %s; /bin/echo %s",
-		 starttoken, env_loc, stoptoken);
+
+	/* Construct the final command */
+	cmdstr = xstrdup_printf("/bin/echo; /bin/echo; /bin/echo; "
+				"/bin/echo %s; %s; /bin/echo %s",
+				starttoken, env_loc, stoptoken);
 	xfree(stepd_path);
 
 	if (pipe(fildes) < 0) {
@@ -2116,10 +2260,17 @@ char **env_array_user_default(const char *username, int timeout, int mode,
 		return NULL;
 	}
 
-	child_args.mode = mode;
 	child_args.fildes = fildes;
 	child_args.username = username;
 	child_args.cmdstr = cmdstr;
+	child_args.tmp_env = env_array_create();
+	child_args.perform_mount = true;
+	env_array_overwrite(&child_args.tmp_env, "ENVIRONMENT", "BATCH");
+	if (getrlimit(RLIMIT_NOFILE, &rlim) < 0) {
+		error("getrlimit(RLIMIT_NOFILE): %m");
+		rlim.rlim_cur = 4096;
+	}
+	child_args.rlimit = rlim.rlim_cur;
 
 #if defined(__APPLE__) || defined(__FreeBSD__) || defined(__NetBSD__)
 	child = fork();
@@ -2130,11 +2281,30 @@ char **env_array_user_default(const char *username, int timeout, int mode,
 	if (child == 0)
 		_child_fn(&child_args);
 #else
-	if ((child = _clone_env_child(&child_args)) == -1) {
-		fatal("clone: %m");
-		return NULL;
+	/*
+	 * Since we will be using namespaces in the clone calls (CLONE_NEWPID,
+	 * CLONE_NEWNS), we need to know if they are disabled . If they are,
+	 * we must fall back to fork and warn the user about the risks.
+	 */
+	if (_ns_disabled()) {
+		warning("%s: pid or mnt namespaces are disabled, avoiding clone and falling back to fork. This can produce orphan/unconstrained processes!",
+			__func__);
+		child_args.perform_mount = false;
+		child = fork();
+		if (child == -1) {
+			fatal("fork: %m");
+			return NULL;
+		}
+		if (child == 0)
+			_child_fn(&child_args);
+	} else {
+		if ((child = _clone_env_child(&child_args)) == -1) {
+			fatal("clone: %m");
+			return NULL;
+		}
 	}
 #endif
+	xfree(cmdstr);
 	close(fildes[1]);
 	if ((fval = fcntl(fildes[0], F_GETFL, 0)) < 0)
 		error("fcntl(F_GETFL) failed: %m");
@@ -2146,8 +2316,6 @@ char **env_array_user_default(const char *username, int timeout, int mode,
 	ufds.events = POLLIN;
 
 	/* Read all of the output from /bin/su into buffer */
-	if (timeout == 0)
-		timeout = slurm_conf.get_env_timeout;	/* != 0 test above */
 	found = 0;
 	buf_read = 0;
 	buffer = xmalloc(ENV_BUFSIZE);
@@ -2158,7 +2326,7 @@ char **env_array_user_default(const char *username, int timeout, int mode,
 		timeleft -= (now.tv_usec - begin.tv_usec) / 1000;
 		if (timeleft <= 0) {
 			verbose("timeout waiting for "SUCMD" to complete");
-			kill(-child, 9);
+			kill(child, 9);
 			break;
 		}
 		if ((rc = poll(&ufds, 1, timeleft)) <= 0) {
@@ -2198,8 +2366,10 @@ char **env_array_user_default(const char *username, int timeout, int mode,
 		}
 	}
 	close(fildes[0]);
+	env_array_free(child_args.tmp_env);
+
 	for (config_timeout=0; ; config_timeout++) {
-		kill(-child, SIGKILL);	/* Typically a no-op */
+		kill(child, SIGKILL); /* Typically a no-op */
 		if (config_timeout)
 			sleep(1);
 		if (waitpid(child, &rc, WNOHANG) > 0)
@@ -2218,7 +2388,7 @@ char **env_array_user_default(const char *username, int timeout, int mode,
 	if (!found) {
 		error("Failed to load current user environment variables");
 		xfree(buffer);
-		return no_cache ? _load_env_cache(username) : NULL;
+		return NULL;
 	}
 
 	/* First look for the start token in the output */
@@ -2235,7 +2405,7 @@ char **env_array_user_default(const char *username, int timeout, int mode,
 	if (!found) {
 		error("Failed to get current user environment variables");
 		xfree(buffer);
-		return no_cache ? _load_env_cache(username) : NULL;
+		return NULL;
 	}
 
 	/* Process environment variables until we find the stop token */
@@ -2275,7 +2445,7 @@ char **env_array_user_default(const char *username, int timeout, int mode,
 	if (!found) {
 		error("Failed to get all user environment variables");
 		env_array_free(env);
-		return no_cache ? _load_env_cache(username) : NULL;
+		return NULL;
 	}
 
 	return env;
@@ -2284,8 +2454,14 @@ char **env_array_user_default(const char *username, int timeout, int mode,
 static void _set_ext_launcher_hydra(char ***dest, char *b_env, char *extra)
 {
 	char *bootstrap = getenv(b_env);
+	bool disabled_slurm_hydra_bootstrap = false;
 
-	if (!bootstrap || !xstrcmp(bootstrap, "slurm")) {
+	if (slurm_conf.mpi_params &&
+	    xstrstr(slurm_conf.mpi_params,"disable_slurm_hydra_bootstrap"))
+		disabled_slurm_hydra_bootstrap = true;
+
+	if ((!bootstrap && !disabled_slurm_hydra_bootstrap) ||
+	    !xstrcmp(bootstrap, "slurm")) {
 		env_array_append(dest, b_env, "slurm");
 		env_array_append(dest, extra, "--external-launcher");
 	}
@@ -2447,13 +2623,7 @@ extern void env_merge_filter(slurm_opt_t *opt, job_desc_msg_t *desc)
 	}
 	xfree(tmp);
 
-	for (i = 0; environ[i]; i++) {
-		if (xstrncmp("SLURM_", environ[i], 6))
-			continue;
-		save_env[0] = environ[i];
-		env_array_merge(&desc->environment,
-				(const char **)save_env);
-	}
+	env_array_merge_slurm_spank(&desc->environment, (const char **)environ);
 }
 
 extern char **env_array_exclude(const char **env, const regex_t *regex)
@@ -2471,3 +2641,24 @@ extern char **env_array_exclude(const char **env, const regex_t *regex)
 
 	return purged;
 }
+
+extern void set_prio_process_env(void)
+{
+        int retval;
+
+        errno = 0; /* needed to detect a real failure since prio can be -1 */
+
+        if ((retval = getpriority(PRIO_PROCESS, 0)) == -1)  {
+                if (errno) {
+                        error("getpriority(PRIO_PROCESS): %m");
+                        return;
+                }
+        }
+
+        if (setenvf(NULL, "SLURM_PRIO_PROCESS", "%d", retval) < 0) {
+                error("unable to set SLURM_PRIO_PROCESS in environment");
+                return;
+        }
+
+        debug("propagating SLURM_PRIO_PROCESS=%d", retval);
+}
