@@ -83,6 +83,7 @@ strong_alias(xfree_struct_group_array, slurm_xfree_struct_group_array);
 strong_alias(stepd_gethostbyname, slurm_stepd_gethostbyname);
 strong_alias(xfree_struct_hostent, slurm_xfree_struct_hostent);
 strong_alias(stepd_get_namespace_fd, slurm_stepd_get_namespace_fd);
+strong_alias(stepd_get_namespace_fds, slurm_stepd_get_namespace_fds);
 
 /*
  * Should be called when a connect() to a socket returns ECONNREFUSED.
@@ -197,10 +198,14 @@ _guess_nodename(void)
 		nodename = slurm_conf_get_nodename("localhost");
 	/*
 	 * If nothing above has given us a name, just return what
-	 * gethostname_short. This is helpful for dynamic nodes.
+	 * gethostname() returns. This is helpful for dynamic nodes whose names
+	 * contain dots.
 	 */
-	if (!nodename)
+	if (!nodename) {
+		if (gethostname(host, sizeof(host)) != 0)
+			return NULL;
 		nodename = xstrdup(host);
+	}
 
 	return nodename;
 }
@@ -220,6 +225,8 @@ extern int stepd_connect(const char *directory, const char *nodename,
 			 slurm_step_id_t *step_id,
 			 uint16_t *protocol_version)
 {
+	char *alloc_dir = NULL;
+	const char *dir = directory;
 	int req = SLURM_PROTOCOL_VERSION;
 	int fd = -1;
 	int rc;
@@ -234,15 +241,16 @@ extern int stepd_connect(const char *directory, const char *nodename,
 	}
 	if (directory == NULL) {
 		slurm_conf_t *cf = slurm_conf_lock();
-		directory = slurm_conf_expand_slurmd_path(cf->slurmd_spooldir,
+		alloc_dir = slurm_conf_expand_slurmd_path(cf->slurmd_spooldir,
 							  nodename, NULL);
+		dir = alloc_dir;
 		slurm_conf_unlock();
 	}
 
 	/* Connect to the step */
-	fd = _step_connect(directory, nodename, step_id);
+	fd = _step_connect(dir, nodename, step_id);
 	if (fd == -1)
-		goto fail1;
+		goto cleanup;
 
 	safe_write(fd, &req, sizeof(int));
 	safe_read(fd, &rc, sizeof(int));
@@ -251,13 +259,13 @@ extern int stepd_connect(const char *directory, const char *nodename,
 	else if (rc)
 		*protocol_version = rc;
 
-	xfree(local_nodename);
-	return fd;
+	goto cleanup;
 
 rwfail:
 	fd_close(&fd);
-fail1:
+cleanup:
 	xfree(local_nodename);
+	xfree(alloc_dir);
 	return fd;
 }
 
@@ -686,6 +694,8 @@ extern list_t *stepd_available(const char *directory, const char *nodename)
 	regex_t re;
 	struct stat stat_buf;
 	char *local_nodename = NULL;
+	char *alloc_dir = NULL;
+	const char *dir = directory;
 
 	if (nodename == NULL) {
 		if (!(local_nodename = _guess_nodename())) {
@@ -696,8 +706,9 @@ extern list_t *stepd_available(const char *directory, const char *nodename)
 	}
 	if (directory == NULL) {
 		slurm_conf_t *cf = slurm_conf_lock();
-		directory = slurm_conf_expand_slurmd_path(
-			cf->slurmd_spooldir, nodename, NULL);
+		alloc_dir = slurm_conf_expand_slurmd_path(cf->slurmd_spooldir,
+							  nodename, NULL);
+		dir = alloc_dir;
 		slurm_conf_unlock();
 	}
 
@@ -708,15 +719,15 @@ extern list_t *stepd_available(const char *directory, const char *nodename)
 	/*
 	 * Make sure that "directory" exists and is a directory.
 	 */
-	if (stat(directory, &stat_buf) < 0) {
-		error("Domain socket directory %s: %m", directory);
+	if (stat(dir, &stat_buf) < 0) {
+		error("Domain socket directory %s: %m", dir);
 		goto done;
 	} else if (!S_ISDIR(stat_buf.st_mode)) {
-		error("%s is not a directory", directory);
+		error("%s is not a directory", dir);
 		goto done;
 	}
 
-	if ((dp = opendir(directory)) == NULL) {
+	if ((dp = opendir(dir)) == NULL) {
 		error("Unable to open directory: %m");
 		goto done;
 	}
@@ -728,10 +739,10 @@ extern list_t *stepd_available(const char *directory, const char *nodename)
 		if (!_sockname_regex(&re, ent->d_name, &step_id)) {
 			debug4("found %ps", &step_id);
 			loc = xmalloc(sizeof(step_loc_t));
-			loc->directory = xstrdup(directory);
+			loc->directory = xstrdup(dir);
 			loc->nodename = xstrdup(nodename);
 			loc->step_id = step_id;
-			list_append(l, (void *)loc);
+			list_append(l, (void *) loc);
 		}
 	}
 
@@ -739,6 +750,7 @@ extern list_t *stepd_available(const char *directory, const char *nodename)
 done:
 	xfree(local_nodename);
 	regfree(&re);
+	xfree(alloc_dir);
 	return l;
 }
 
