@@ -46,15 +46,10 @@ static int _set_cond(int *start, int argc, char **argv,
 	int command_len = 0;
 
 	for (i=(*start); i<argc; i++) {
-		end = parse_option_end(argv[i]);
-		if (!end)
-			command_len=strlen(argv[i]);
-		else {
-			command_len=end-1;
-			if (argv[i][end] == '=') {
-				end++;
-			}
-		}
+		int op_type;
+		end = parse_option_end(argv[i], &op_type, &command_len);
+		if (!common_verify_option_syntax(argv[i], op_type, false))
+			continue;
 
 		if (!xstrncasecmp(argv[i], "Set", MAX(command_len, 3))) {
 			i--;
@@ -114,20 +109,12 @@ static int _set_rec(int *start, int argc, char **argv,
 	int end = 0;
 	int command_len = 0;
 	int option = 0;
+	bool allow_option = false;
 
 	xassert(fed);
 
 	for (i=(*start); i<argc; i++) {
-		end = parse_option_end(argv[i]);
-		if (!end)
-			command_len=strlen(argv[i]);
-		else {
-			command_len=end-1;
-			if (argv[i][end] == '=') {
-				option = (int)argv[i][end-1];
-				end++;
-			}
-		}
+		end = parse_option_end(argv[i], &option, &command_len);
 
 		if (!xstrncasecmp (argv[i], "Where", MAX(command_len, 5))) {
 			i--;
@@ -142,6 +129,7 @@ static int _set_rec(int *start, int argc, char **argv,
 				slurm_addto_char_list(name_list, argv[i]+end);
 		} else if (!xstrncasecmp(argv[i], "Clusters",
 					 MAX(command_len, 2))) {
+			allow_option = true;
 			char *name = NULL;
 			list_itr_t *itr;
 
@@ -179,6 +167,7 @@ static int _set_rec(int *start, int argc, char **argv,
 			set = 1;
 		} else if (!xstrncasecmp(argv[i], "Flags",
 					 MAX(command_len, 2))) {
+			allow_option = true;
 			fed->flags = str_2_federation_flags(argv[i]+end,
 								   option);
 			if (fed->flags == FEDERATION_FLAG_NOTSET) {
@@ -199,12 +188,15 @@ static int _set_rec(int *start, int argc, char **argv,
 			} else
 				set = 1;
 		} else {
+			allow_option = true;
 			exit_code = 1;
 			fprintf(stderr,
 				" Unknown option: %s\n"
 				" Use keyword 'where' to modify condition\n",
 				argv[i]);
 		}
+
+		common_verify_option_syntax(argv[i], option, allow_option);
 	}
 
 	(*start) = i;
