@@ -1504,7 +1504,7 @@ x11_fail:
 			step->oom_error = true;
 
 	/* Lock to not collide with the _x11_signal_handler thread. */
-	auth_setuid_lock();
+	auth_context_lock();
 
 	/*
 	 * This function below calls jobacct_gather_fini(). For the case of
@@ -1515,7 +1515,7 @@ x11_fail:
 	acct_gather_profile_fini();
 	task_g_post_step(step);
 
-	auth_setuid_unlock();
+	auth_context_unlock();
 
 fail1:
 	conmgr_add_work_fifo(_x11_signal_handler, NULL);
@@ -2228,6 +2228,7 @@ static int _fork_all_tasks(bool *io_initialized)
 			goto fail4;
 		} else if ((pid = _exec_wait_get_pid(ei)) == 0) { /* child */
 			int rc;
+			sigset_t mask;
 
 			/*
 			 *  Destroy exec_wait_list in the child.
@@ -2237,6 +2238,17 @@ static int _fork_all_tasks(bool *io_initialized)
 			 */
 			FREE_NULL_LIST(exec_wait_list);
 
+			/*
+			 * Unblock SIGCHLD since this is blocked early by the
+			 * slurmstepd
+			 */
+			sigemptyset(&mask);
+			sigaddset(&mask, SIGCHLD);
+			if (pthread_sigmask(SIG_UNBLOCK, &mask, NULL) == -1) {
+				error("pthread_sigmask() failed: %m");
+				_exit(1);
+			}
+
 			/* jobacctinfo_endpoll();
 			 * closing jobacct files here causes deadlock */
 
