@@ -253,6 +253,7 @@ extern int read_slurmdbd_conf(void)
 		s_p_get_boolean(&tmp_bool, "AllowNoDefAcct", tbl);
 		if (tmp_bool)
 			slurmdbd_conf->flags |= DBD_CONF_FLAG_ALLOW_NO_DEF_ACCT;
+		tmp_bool = false;
 		s_p_get_boolean(&tmp_bool, "AllResourcesAbsolute", tbl);
 		if (tmp_bool)
 			slurmdbd_conf->flags |= DBD_CONF_FLAG_ALL_RES_ABS;
@@ -351,11 +352,13 @@ extern int read_slurmdbd_conf(void)
 					SLURMDB_PURGE_MONTHS;
 		}
 
+		tmp_bool = false;
 		s_p_get_boolean(&tmp_bool, "DisableCoordDBD", tbl);
 		if (tmp_bool)
 			slurmdbd_conf->flags |=
 				DBD_CONF_FLAG_DISABLE_COORD_DBD;
 
+		tmp_bool = false;
 		s_p_get_boolean(&tmp_bool, "DisableArchiveCommands", tbl);
 		if (tmp_bool)
 			slurmdbd_conf->flags |=
@@ -824,7 +827,7 @@ extern list_t *dump_config(void)
 	add_key_pair(my_list, "BOOT_TIME", "%s", tmp_ptr);
 	xfree(tmp_ptr);
 
-	add_key_pair_bool(my_list, "CommitDelay", slurmdbd_conf->commit_delay);
+	add_key_pair(my_list, "CommitDelay", "%u", slurmdbd_conf->commit_delay);
 
 	add_key_pair(my_list, "CommunicationParameters", "%s",
 		     slurm_conf.comm_params);
